﻿using ProjectMonorail.Generics;
using ProjectMonorail.MovementStrategy;
using ProjectMonorail.DrawingObjects;

namespace ProjectMonorail
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawingMonorail
    /// </summary>
    public partial class FormMonorailCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly MonorailsGenericCollection<DrawingMonorail, DrawingObjectMonorail> _monorails;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormMonorailCollection()
        {
            InitializeComponent();
            _monorails = new MonorailsGenericCollection<DrawingMonorail, DrawingObjectMonorail>(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddMonorail_Click(object sender, EventArgs e)
        {
            FormMonorail form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                if (_monorails + form.SelectedMonorail != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = _monorails.ShowMonorails();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }

        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRemoveMonorail_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }

            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);

            if (_monorails - pos)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = _monorails.ShowMonorails();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRefreshCollection_Click(object sender, EventArgs e)
        {
            pictureBoxCollection.Image = _monorails.ShowMonorails();
        }
    }
}
